/* 
 * File:   main.c
 * Author: take
 *
 * Created on 2013/05/05, 19:21
 */
#include <xc.h>
#include "lcd.h"
#define _XTAL_FREQ 20000000	// __delay_ms()用クロック定義 (20MHz駆動)

#pragma config FOSC = HS
#pragma config WDTE = OFF

#define LCD_E  RA0
#define	LCD_RW RA1
#define	LCD_RS RA2
#define	buffsize 40
#define	RS_cmd  0xFE
#define	EEsave  0

#define Delay_ms(ms)    __delay_ms(ms)
#define LCD_Busy_wait() // dummy
#define LCD_send4(c)    // dummy
#define LCD_data(c)     lcd_putc(c)
#define LCD_cmd(c)      lcd_write_cmd(c)
#define LCD_init()      lcd_init()

static void pic_init();
static void sw_RA3_on();
static void bufLCD();

static unsigned char buff[buffsize];
static unsigned char buffR;		// バッファ読み込みポインタ
static unsigned char buffW;		// バッファ書き込みポインタ
static unsigned char SW_RS;		// LCD書き込みモード
static unsigned char comm_stat;	// 通信状態
	// 0:無通信
	// 1:スタートビット受信，半ビット時間待ち
	// 2:データビット受信中
	// 3:データビット受信完了，ストップビット待ち

static unsigned char comm_char;	// 受信データ
static unsigned char comm_bit;	// 受信データのビット数
static unsigned char T0val = 127;		// TMR0設定値
static unsigned char sw_RA3;	// RA3 SW状態
static unsigned char speed = 2;		// 通信速度(1:4800,2:9600,3:19200)

// =================== 割り込み処理 ====================
static void interrupt intr(void) {
	if (T0IF) {			// TMR0割り込み？
		TMR0 = 256 - T0val;	// TMR0
		T0IF = 0;			// TMR0割り込みクリア
		RB4 ^= 1;	// DEBUG
		switch (comm_stat) {
                case 1:		// ■スタートビット受信開始，半ビット待ち
                        comm_stat = 2;	// データビット受信中
                        comm_bit = 8;	// 8ビット受信開始
                        comm_char = 0;  // 0 clear;
                        break;
                case 2:		// ■データビット受信中
                        comm_char >>= 1;
                        if (RB7) {
                                comm_char |= 0x80;
                        }
                        comm_bit--;
                        if (!(comm_bit)) {
                                buff[buffW] = comm_char;
                                comm_char = 0;
                                comm_stat = 3;	// データビット受信完了，ストップビット待ち

                                // バッファ書き込みポインタの更新
                                buffW++;
                                if (buffW == buffsize) {	// バッファポインタは最終位置？
                                        buffW = 0;
                                }

                        }
                        break;
                case 3:		// ■ストップビット待ち
                        comm_stat = 0;	// 無通信状態
                        if (!RB7) {
                                RB5 = 0;	// LED消灯
                        }
                        RB4 = 0;	// DEBUG
                        T0IE = 0;
			RBIE = 1;
			RBIF = 0;
                        break;
		}
	}
        else
	if (RBIF) {	// PORTBピン変化割り込み？
		RB5 ^= 1;
		if (!RB7) {				// スタートビット？
			if (comm_stat == 0) {	// ■無通信状態
				comm_stat = 1;	// スタートビット受信開始，半ビット待ち
				comm_bit = 8;	// 8ビット受信開始
                                comm_char = 0;
				RB4 = 0;	// DEBUG
				TMR0 = 256 - (T0val>>1);// TMR0
				T0IF = 0;
				T0IE = 1;
                                //RBIE = 0;
			}
		}
		RBIF = 0;	// PORTBピン変化割り込みクリア
	}
}

// ==================== メイン処理 ===========================
main()	{

	pic_init();			// PICの初期化

	LCD_init();			// LCDの初期化
	// EEspeed();
        lcd_puts("Start serLCD");
	buffW = 0;
	buffR = 0;
	SW_RS = 1;			// LCD Data mode

	Delay_ms(250);
	Delay_ms(250);
	comm_stat = 0;
	RB7 = 0;
	INTCON = 0x88;		// GIE=1, RBIE=1

	while(1) {
		if	(buffW != buffR) {
			bufLCD();
		}
		if	(RA3) {			// RA3:1 (SW off)
			sw_RA3 = 0;
		} else {			// RA3:0 (SW on)
			sw_RA3_on();
		}
	}
}

// ==================== PICの初期化 ===========================
static void pic_init() {
	OPTION_REG = 0x01;	// TMR0プリスケーラ:4
	TRISA = 0xF8;		// RA0-2:out
	TRISB = 0x80;		// RB0-6:out
	PORTA = 0x00;
	PORTB = 0x00;
	INTCON = 0x00;		// GIE:0 ,T0IE:0 ,RBIE:0
}

// =============== bufferの内容をLCDに出力 ===================
static void bufLCD() {
	unsigned char c;
	c = buff[buffR];
	switch (c) {
		case RS_cmd:
			SW_RS = 0;		// Command mode set
			break;
		default:
			if (SW_RS) {LCD_data(c);}
			else {
                            LCD_cmd(c);
                            SW_RS = 1;
                        }
	}
	buffR++;
	if	(buffR == buffsize) {	// バッファポインタは最終位置？
		buffR = 0;
	}
}

// ==================== RA3スイッチＯＮ処理 ==============
//				スピード切り替え
static void sw_RA3_on() {
	Delay_ms(20);		// チャタリングキャンセル
	if	(!RA3) {		// RA3 on 確定
		if	(!(sw_RA3)) {
			sw_RA3 = 1;
			// if	(!(speed)) {speed = 6;}
			LCD_cmd(0x01);		// LCDクリア
			//speedDSP();			// スピードをLCDに表示
			comm_stat = 0;		// 無通信状態
			//EEwrite(EEsave,speed);
			//speed--;
		}
	}
}
